@@ -13,14 +13,14 @@
 #'
 #' In most cases we do not need to call these hooks explicitly, and they were
 #' designed to be used internally. Sometimes we may not be able to record R
-#' plots using \code{\link[grDevices]{recordPlot}}, and we can make use of these
-#' hooks to insert graphics output in the output document; see
+#' plots using \code{grDevices::\link{recordPlot}()}, and we can make use of
+#' these hooks to insert graphics output in the output document; see
 #' \code{\link{hook_plot_custom}} for details.
 #' @param x Filename for the plot (a character string).
 #' @param options A list of the current chunk options.
 #' @rdname hook_plot
 #' @return A character string of code, with plot filenames wrapped.
-#' @references \url{https://yihui.name/knitr/hooks/}
+#' @references \url{https://yihui.org/knitr/hooks/}
 #' @seealso \code{\link{hook_plot_custom}}
 #' @export
 #' @examples # this is what happens for a chunk like this
@@ -114,7 +114,7 @@ hook_plot_tex = function(x, options) {
     }
     # Add subfloat code if needed
     if (usesub) {
-      sub1 = sprintf('\\subfloat[%s%s]{', subcap, create_label(lab, fig.cur, latex = TRUE))
+      sub1 = sprintf('\\subfloat[%s%s]{', subcap, create_label(lab, '-', fig.cur, latex = TRUE))
       sub2 = '}'
       sep.cur = fig.sep[fig.cur]; if (is.na(sep.cur)) sep.cur = NULL
     }
@@ -129,7 +129,7 @@ hook_plot_tex = function(x, options) {
       scap = if (is.null(scap) || is.na(scap)) '' else sprintf('[%s]', scap)
       cap = if (cap == '') '' else sprintf(
         '\\caption%s{%s}%s\n', scap, cap,
-        create_label(lab, if (mcap) fig.cur, latex = TRUE)
+        create_label(lab, if (mcap) c('-', fig.cur), latex = TRUE)
       )
       fig2 = sprintf('%s\\end{%s}\n', cap, options$fig.env)
     }
@@ -249,7 +249,7 @@ hook_plot_tex = function(x, options) {
 #' @rdname output_hooks
 #' @return \code{NULL}; corresponding hooks are set as a side effect
 #' @export
-#' @references See output hooks in \url{https://yihui.name/knitr/hooks/}.
+#' @references See output hooks in \url{https://yihui.org/knitr/hooks/}.
 #'
 #'   Jekyll and Liquid:
 #'   \url{https://github.com/jekyll/jekyll/wiki/Liquid-Extensions};
@@ -342,7 +342,7 @@ render_listings = function() {
 #'   you to use the default output hooks for LaTeX (not Sweave or listings), and
 #'   every figure/table environment must have a label.
 #' @export
-#' @references \url{https://yihui.name/knitr/hooks/}
+#' @references \url{https://yihui.org/knitr/hooks/}
 #' @examples \dontrun{knit_hooks$set(document = hook_movecode)}
 #' # see example 103 at https://github.com/yihui/knitr-examples
 hook_movecode = function(x) {
