@@ -4,8 +4,8 @@
 #' list of patterns, evaluates the code and writes the output in another file.
 #' It can also tangle R source code from the input document (\code{purl()} is a
 #' wrapper to \code{knit(..., tangle = TRUE)}). The \code{knitr.purl.inline}
-#' option can be used to also tangle the code of inline expressions (disabled
-#' by default).
+#' option can be used to also tangle the code of inline expressions (disabled by
+#' default).
 #'
 #' For most of the time, it is not necessary to set any options outside the
 #' input document; in other words, a single call like
@@ -34,9 +34,9 @@
 #' \code{apat$rnw}, \samp{tex} uses the list \code{apat$tex}, \samp{brew} uses
 #' \code{apat$brew} and HTML files use \code{apat$html}; for unkown extensions,
 #' the content of the input document is matched against all pattern lists to
-#' automatically determine which pattern list is being used. You can also manually
-#' set the pattern list using the \code{\link{knit_patterns}} object or the
-#' \code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
+#' automatically determine which pattern list is being used. You can also
+#' manually set the pattern list using the \code{\link{knit_patterns}} object or
+#' the \code{\link{pat_rnw}} series functions in advance and \pkg{knitr} will
 #' respect the setting.
 #'
 #' According to the output format (\code{opts_knit$get('out.format')}), a set of
@@ -57,15 +57,15 @@
 #' etc.
 #' @param input Path to the input file.
 #' @param output Path to the output file for \code{knit()}. If \code{NULL}, this
-#'   function will try to guess a default, which will be under the current working
-#'   directory.
+#'   function will try to guess a default, which will be under the current
+#'   working directory.
 #' @param tangle Boolean; whether to tangle the R code from the input file (like
-#'   \code{\link[utils]{Stangle}}).
-#' @param text A character vector. This is an alternative way to provide the input
-#'   file.
+#'   \code{utils::\link{Stangle}}).
+#' @param text A character vector. This is an alternative way to provide the
+#'   input file.
 #' @param quiet Boolean; suppress the progress bar and messages?
-#' @param envir Environment in which code chunks are to be evaluated,
-#'   for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
+#' @param envir Environment in which code chunks are to be evaluated, for
+#'   example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
 #'   \code{\link{globalenv}()}).
 #' @param encoding Encoding of the input file; always assumed to be UTF-8 (i.e.,
 #'   this argument is effectively ignored).
@@ -82,20 +82,20 @@
 #'   \code{\link{opts_knit}$set(root.dir = ...)} so all paths in code chunks are
 #'   relative to this \code{root.dir}. It is not recommended to change the
 #'   working directory via \code{\link{setwd}()} in a code chunk, because it may
-#'   lead to terrible consequences (e.g. figure and cache files may be written to
-#'   wrong places). If you do use \code{setwd()}, please note that \pkg{knitr}
-#'   will always restore the working directory to the original one. Whenever you
-#'   feel confused, print \code{getwd()} in a code chunk to see what the working
-#'   directory really is.
+#'   lead to terrible consequences (e.g. figure and cache files may be written
+#'   to wrong places). If you do use \code{setwd()}, please note that
+#'   \pkg{knitr} will always restore the working directory to the original one.
+#'   Whenever you feel confused, print \code{getwd()} in a code chunk to see
+#'   what the working directory really is.
 #'
 #'   If the \code{output} argument is a file path, it is strongly recommended to
 #'   be in the current working directory (e.g. \file{foo.tex} instead of
 #'   \file{somewhere/foo.tex}), especially when the output has external
 #'   dependencies such as figure files. If you want to write the output to a
-#'   different directory, it is recommended to set the working directory to
-#'   that directory before you knit a document. For example, if the source
-#'   document is \file{foo.Rmd} and the expected output is \file{out/foo.md},
-#'   you can write \code{setwd('out/'); knit('../foo.Rmd')} instead of
+#'   different directory, it is recommended to set the working directory to that
+#'   directory before you knit a document. For example, if the source document
+#'   is \file{foo.Rmd} and the expected output is \file{out/foo.md}, you can
+#'   write \code{setwd('out/'); knit('../foo.Rmd')} instead of
 #'   \code{knit('foo.Rmd', 'out/foo.md')}.
 #'
 #'   N.B. There is no guarantee that the R script generated by \code{purl()} can
@@ -425,7 +425,7 @@ knit_log = new_defaults()  # knitr log for errors, warnings and messages
 
 #' Wrap evaluated results for output
 #'
-#' @param x output from \code{\link[evaluate]{evaluate}}
+#' @param x output from \code{evaluate::\link{evaluate}()}
 #' @param options List of options used to control output
 #' @noRd
 wrap = function(x, options = list(), ...) {
@@ -541,19 +541,19 @@ wrap.recordedplot = function(x, options) {
       dir.create(dirname(name), recursive = TRUE)
   )
   # vectorize over dev, ext and dpi: save multiple versions of the plot
-  file = mapply(
+  files = mapply(
     save_plot, width = options$fig.width, height = options$fig.height,
     dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
     MoreArgs = list(plot = x, name = name, options = options), SIMPLIFY = FALSE
-  )[[1]]
+  )
+  opts_knit$append(plot_files = unlist(files))
   if (options$fig.show == 'hide') return('')
-  in_base_dir(knit_hooks$get('plot')(file, reduce_plot_opts(options)))
+  in_base_dir(run_hook_plot(files[[1]], reduce_plot_opts(options)))
 }
 
 #' @export
 wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE) {
   if (options$fig.show == 'hide') return('')
-  hook_plot = knit_hooks$get('plot')
   # remove the automatically set out.width when fig.retina is set, otherwise the
   # size of external images embedded via include_graphics() will be set to
   # fig.width * dpi in fix_options()
@@ -563,19 +563,20 @@ wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE)
     if (length(w1) * length(w2) == 1 && is.numeric(w1) && w1 == w2)
       options['out.width'] = list(NULL)
   }
+  options$fig.num = options$fig.num %n% length(x)
   dpi = attr(x, 'dpi') %n% options$dpi
+  hook = knit_hooks$get('plot')
   paste(unlist(lapply(seq_along(x), function(i) {
     options$fig.cur = plot_counter()
     if (is.null(options[['out.width']]))
       options['out.width'] = list(raster_dpi_width(x[i], dpi))
-    hook_plot(x[i], reduce_plot_opts(options))
+    hook(x[i], reduce_plot_opts(options))
   })), collapse = '')
 }
 
 #' @export
 wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
   ext = x$extension
-  hook_plot = knit_hooks$get('plot')
   in_base_dir({
     i = plot_counter()
     if (is.null(f <- x$file)) {
@@ -583,15 +584,21 @@ wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
       dir.create(dirname(f), recursive = TRUE, showWarnings = FALSE)
       writeBin(x$image, f, useBytes = TRUE)
     }
-    # crop white margins
-    if (isTRUE(options$crop)) in_dir(dirname(f), plot_crop(basename(f)))
     options$fig.cur = i
     options = reduce_plot_opts(options)
     if (!is.null(x$url) && is.null(options$fig.link)) options$fig.link = x$url
-    hook_plot(f, options)
+    run_hook_plot(f, options)
   })
 }
 
+# record plot filenames in opts_knit$get('plot_files'), including those from R
+# code and auto screenshots of HTML widgets, etc. Then run the plot hook.
+run_hook_plot = function(x, options) {
+  opts_knit$append(plot_files = x)
+  hook = knit_hooks$get('plot')
+  hook(x, options)
+}
+
 #' @export
 wrap.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE) {
   options$fig.cur = plot_counter()
