@@ -1,18 +1,27 @@
 #' Create tables in LaTeX, HTML, Markdown and reStructuredText
 #'
-#' This is a very simple table generator. It is simple by design. It is not
-#' intended to replace any other R packages for making tables.
+#' A very simple table generator, and it is simple by design. It is not intended
+#' to replace any other R packages for making tables. The \code{kable()}
+#' function returns a single table for a single data object, and returns a table
+#' that contains multiple tables if the input object is a list of data objects.
+#' The \code{kables()} function is similar to \code{kable(x)} when \code{x} is a
+#' list of data objects, but \code{kables()} accepts a list of \code{kable()}
+#' values directly instead of data objects (see examples below).
 #'
 #' Missing values (\code{NA}) in the table are displayed as \code{NA} by
 #' default. If you want to display them with other characters, you can set the
 #' option \code{knitr.kable.NA}, e.g. \code{options(knitr.kable.NA = '')} to
 #' hide \code{NA} values.
-#' @param x An R object, typically a matrix or data frame.
+#' @param x For \code{kable()}, \code{x} is an R object, which is typically a
+#'   matrix or data frame. For \code{kables()}, a list with each element being a
+#'   returned value from \code{kable()}.
 #' @param format A character string. Possible values are \code{latex},
-#'   \code{html}, \code{markdown}, \code{pandoc}, and \code{rst}; this will be
-#'   automatically determined if the function is called within \pkg{knitr}; it
-#'   can also be set in the global option \code{knitr.table.format}. If
-#'   \code{format} is a function, it must return a character string.
+#'   \code{html}, \code{pipe} (Pandoc's pipe tables), \code{simple} (Pandoc's
+#'   simple tables), and \code{rst}. The value of this argument will be
+#'   automatically determined if the function is called within a \pkg{knitr}
+#'   document. The \code{format} value can also be set in the global option
+#'   \code{knitr.table.format}. If \code{format} is a function, it must return a
+#'   character string.
 #' @param digits Maximum number of digits for numeric columns, passed to
 #'   \code{round()}. This can also be a vector of length \code{ncol(x)}, to set
 #'   the number of digits for individual columns.
@@ -40,99 +49,73 @@
 #'   \pkg{kableExtra}, and \pkg{tables} for HTML and LaTeX tables, and
 #'   \pkg{ascii} and \pkg{pander} for different flavors of markdown output and
 #'   some advanced features and table styles.
-#' @note The tables for \code{format = 'markdown'} also work for Pandoc when the
-#'   \code{pipe_tables} extension is enabled (this is the default behavior for
-#'   Pandoc >= 1.10).
-#'
-#'   When using \code{kable()} as a \emph{top-level} expression, you do not need
-#'   to explicitly \code{print()} it due to R's automatic implicit printing.
-#'   When it is wrapped inside other expressions (such as a \code{\link{for}}
-#'   loop), you must explicitly \code{print(kable(...))}.
+#' @note When using \code{kable()} as a \emph{top-level} expression, you do not
+#'   need to explicitly \code{print()} it due to R's automatic implicit
+#'   printing. When it is wrapped inside other expressions (such as a
+#'   \code{\link{for}} loop), you must explicitly \code{print(kable(...))}.
 #' @references See
 #'   \url{https://github.com/yihui/knitr-examples/blob/master/091-knitr-table.Rnw}
 #'    for some examples in LaTeX, but they also apply to other document formats.
 #' @export
-#' @examples  kable(head(iris), format = 'latex')
-#' kable(head(iris), format = 'html')
-#' kable(head(iris), format = 'latex', caption = 'Title of the table')
-#' kable(head(iris), format = 'html', caption = 'Title of the table')
+#' @examples d1 = head(iris); d2 = head(mtcars)
+#' # pipe tables by default
+#' kable(d1)
+#' kable(d2[, 1:5])
+#' # no inner padding
+#' kable(d2, format = 'pipe', padding = 0)
+#' # more padding
+#' kable(d2, format = 'pipe', padding = 2)
+#' kable(d1, format = 'latex')
+#' kable(d1, format = 'html')
+#' kable(d1, format = 'latex', caption = 'Title of the table')
+#' kable(d1, format = 'html', caption = 'Title of the table')
 #' # use the booktabs package
 #' kable(mtcars, format = 'latex', booktabs = TRUE)
 #' # use the longtable package
 #' kable(matrix(1000, ncol=5), format = 'latex', digits = 2, longtable = TRUE)
 #' # change LaTeX default table environment
-#' kable(head(iris), format = "latex", caption = "My table", table.envir='table*')
+#' kable(d1, format = "latex", caption = "My table", table.envir='table*')
 #' # add some table attributes
-#' kable(head(iris), format = 'html', table.attr = 'id="mytable"')
+#' kable(d1, format = 'html', table.attr = 'id="mytable"')
 #' # reST output
-#' kable(head(mtcars), format = 'rst')
+#' kable(d2, format = 'rst')
 #' # no row names
-#' kable(head(mtcars), format = 'rst', row.names = FALSE)
-#' # R Markdown/Github Markdown tables
-#' kable(head(mtcars[, 1:5]), format = 'markdown')
-#' # no inner padding
-#' kable(head(mtcars), format = 'markdown', padding = 0)
-#' # more padding
-#' kable(head(mtcars), format = 'markdown', padding = 2)
-#' # Pandoc tables
-#' kable(head(mtcars), format = 'pandoc', caption = 'Title of the table')
+#' kable(d2, format = 'rst', row.names = FALSE)
+#' # Pandoc simple tables
+#' kable(d2, format = 'simple', caption = 'Title of the table')
 #' # format numbers using , as decimal point, and ' as thousands separator
 #' x = as.data.frame(matrix(rnorm(60, 1e6, 1e4), 10))
 #' kable(x, format.args = list(decimal.mark = ',', big.mark = "'"))
 #' # save the value
-#' x = kable(mtcars, format = 'html')
+#' x = kable(d2, format = 'html')
 #' cat(x, sep = '\n')
 #' # can also set options(knitr.table.format = 'html') so that the output is HTML
+#'
+#' # multiple tables via either kable(list(x1, x2)) or kables(list(kable(x1), kable(x2)))
+#' kable(list(d1, d2), caption = 'A tale of two tables')
+#' kables(list(kable(d1, align = 'l'), kable(d2)), caption = 'A tale of two tables')
 kable = function(
   x, format, digits = getOption('digits'), row.names = NA, col.names = NA,
   align, caption = NULL, label = NULL, format.args = list(), escape = TRUE, ...
 ) {
 
-  # determine the table format
-  if (missing(format) || is.null(format)) format = getOption('knitr.table.format')
-  if (is.null(format)) format = if (is.null(pandoc_to())) switch(
-    out_format() %n% 'markdown',
-    latex = 'latex', listings = 'latex', sweave = 'latex',
-    html = 'html', markdown = 'markdown', rst = 'rst',
-    stop('table format not implemented yet!')
-  ) else if (isTRUE(opts_knit$get('kable.force.latex')) && is_latex_output()) {
-    # force LaTeX table because Pandoc's longtable may not work well with floats
-    # http://tex.stackexchange.com/q/276699/9128
-    'latex'
-  } else 'pandoc'
-  if (is.function(format)) format = format()
-
+  format = kable_format(format)
   # expand align if applicable
   if (!missing(align) && length(align) == 1L && !grepl('[^lcr]', align))
     align = strsplit(align, '')[[1]]
 
-  # create a label for bookdown if applicable
-  if (is.null(label)) label = opts_current$get('label')
-  if (!is.null(caption) && !is.na(caption)) caption = paste0(
-    create_label('tab:', label, latex = (format == 'latex')), caption
-  )
   if (inherits(x, 'list')) {
-    # if the output is for Pandoc and we want multiple tabular in one table, we
-    # should use the latex format instead, because Pandoc does not support
-    # Markdown in LaTeX yet https://github.com/jgm/pandoc/issues/2453
-    if (format == 'pandoc' && is_latex_output()) format = 'latex'
+    format = kable_format_latex(format)
     res = lapply(
       x, kable, format = format, digits = digits, row.names = row.names,
       col.names = col.names, align = align, caption = NA,
       format.args = format.args, escape = escape, ...
     )
-    res = unlist(lapply(res, one_string))
-    res = if (format == 'latex') {
-      kable_latex_caption(res, caption)
-    } else if (format == 'html' || (format == 'pandoc' && is_html_output())) kable_html(
-      matrix(paste0('\n\n', res, '\n\n'), 1), caption = caption, escape = FALSE,
-      table.attr = 'class="kable_wrapper"'
-    ) else {
-      res = paste(res, collapse = '\n\n')
-      if (format == 'pandoc') kable_pandoc_caption(res, caption) else res
-    }
-    return(structure(res, format = format, class = 'knitr_kable'))
+    return(kables(res, format, caption, label))
   }
+
+  caption = kable_caption(label, caption, format)
+
   if (!is.matrix(x)) x = as.data.frame(x)
   if (identical(col.names, NA)) col.names = colnames(x)
   m = ncol(x)
@@ -159,7 +142,7 @@ kable = function(
     if (!is.null(align)) align = c('l', align)  # left align row names
   }
   n = nrow(x)
-  x = replace_na(to_character(as.matrix(x)), is.na(x))
+  x = replace_na(to_character(x), is.na(x))
   if (!is.matrix(x)) x = matrix(x, nrow = n)
   x = trimws(x)
   colnames(x) = col.names
@@ -173,9 +156,72 @@ kable = function(
   structure(res, format = format, class = 'knitr_kable')
 }
 
+kable_caption = function(label, caption, format) {
+  # create a label for bookdown if applicable
+  if (is.null(label)) label = opts_current$get('label')
+  if (!is.null(caption) && !is.na(caption)) caption = paste0(
+    create_label('tab:', label, latex = (format == 'latex')), caption
+  )
+  caption
+}
+
+# determine the table format
+kable_format = function(format = NULL) {
+  if (missing(format) || is.null(format)) format = getOption('knitr.table.format')
+  if (is.null(format)) format = if (is.null(pandoc_to())) switch(
+    out_format() %n% 'markdown',
+    latex = 'latex', listings = 'latex', sweave = 'latex',
+    html = 'html', markdown = 'pipe', rst = 'rst',
+    stop('table format not implemented yet!')
+  ) else if (isTRUE(opts_knit$get('kable.force.latex')) && is_latex_output()) {
+    # force LaTeX table because Pandoc's longtable may not work well with floats
+    # http://tex.stackexchange.com/q/276699/9128
+    'latex'
+  } else 'pipe'
+  if (is.function(format)) format = format()
+  # backward compatibility with knitr <= v1.28
+  switch(format, pandoc = 'simple', markdown = 'pipe', format)
+}
+
+# if the output is for Pandoc and we want multiple tabular in one table, we
+# should use the latex format instead, because Pandoc does not support Markdown
+# in LaTeX yet https://github.com/jgm/pandoc/issues/2453
+kable_format_latex = function(format) {
+  if (format == 'pipe' && is_latex_output()) 'latex' else format
+}
+
+#' @rdname kable
+#' @export
+kables = function(x, format, caption = NULL, label = NULL) {
+  format = kable_format(format)
+  format = kable_format_latex(format)
+  caption = kable_caption(label, caption, format)
+  # in case `x` contains kable()s, make sure all kable()s use the same default format
+  opts = options(knitr.table.format = format); on.exit(options(opts), add = TRUE)
+  if (!inherits(x, 'list')) stop("'x' must be a list (of kable() values)")
+  res = unlist(lapply(x, one_string))
+  res = if (format == 'latex') {
+    kable_latex_caption(res, caption)
+  } else if (format == 'html' || (format == 'pipe' && is_html_output())) kable_html(
+    matrix(paste0('\n\n', res, '\n\n'), 1), caption = caption, escape = FALSE,
+    table.attr = 'class="kable_wrapper"'
+  ) else {
+    res = paste(res, collapse = '\n\n')
+    if (format == 'pipe') kable_pandoc_caption(res, caption) else res
+  }
+  structure(res, format = format, class = 'knitr_kable')
+}
+
 # convert to character while preserving dim/dimnames attributes
 to_character = function(x) {
+  # TODO: get rid of this hack soon https://github.com/mayoverse/arsenal/pull/299
+  if (check_old_package('arsenal', '3.4.0')) x = as.matrix(x)
   if (is.character(x)) return(x)
+  # format columns individually if x is not a matrix
+  if (!is.matrix(x)) {
+    for (j in seq_len(ncol(x))) x[, j] = format_args(x[, j])
+    x = as.matrix(x)
+  }
   x2 = as.character(x); dim(x2) = dim(x); dimnames(x2) = dimnames(x)
   x2
 }
@@ -348,8 +394,8 @@ kable_rst = function(x, rownames.name = '\\', ...) {
   kable_mark(x, rownames.name = rownames.name)
 }
 
-# actually R Markdown
-kable_markdown = function(x, padding = 1, ...) {
+# Pandoc's pipe table
+kable_pipe = function(x, caption = NULL, padding = 1, ...) {
   if (is.null(colnames(x))) {
     warning('The table should have a header (column names)')
     colnames(x) = rep('', ncol(x))
@@ -362,12 +408,16 @@ kable_markdown = function(x, padding = 1, ...) {
     }
     s
   }, ...)
-  sprintf('|%s|', res)
+  res = sprintf('|%s|', res)
+  # TODO: get rid of this hack soon https://github.com/ropensci/skimr/pull/593
+  if (check_old_package('skimr', '2.1.1')) return(res)
+  kable_pandoc_caption(res, caption)
 }
 
-kable_pandoc = function(x, caption = NULL, padding = 1, ...) {
-  # pandoc's table format cannot create 1-column or 0-row tables
-  tab = if (ncol(x) == 1 || nrow(x) == 0) kable_markdown(
+# Pandoc's simple table
+kable_simple = function(x, caption = NULL, padding = 1, ...) {
+  # simple tables do not support 1-column or 0-row tables
+  tab = if (ncol(x) == 1 || nrow(x) == 0) kable_pipe(
     x, padding = padding, ...
   ) else kable_mark(
     x, c(NA, '-', if (is_blank(colnames(x))) '-' else NA),
